@@ -16,8 +16,9 @@ from typing import Dict, Any
 from dfm_python.nowcast import (
     Nowcast, NowcastResult, NewsDecompResult, BacktestResult,
     DataView, para_const,
-    get_higher_frequency, calculate_backward_date,
+    get_higher_frequency,
 )
+from dfm_python.nowcast.utils import calc_backward_date
 from dfm_python.config import DFMConfig, SeriesConfig, DEFAULT_BLOCK_NAME
 from dfm_python.config.adapter import YamlSource
 from dfm_python.config.results import DFMResult
@@ -269,7 +270,7 @@ class TestNowcastUtilities:
         """Test backward date calculation for nowcasting."""
         target_date = datetime(2020, 3, 31)
         # calculate_backward_date has different signature
-        backward_date = calculate_backward_date(target_date, step=1, freq="m")
+        backward_date = calc_backward_date(target_date, step=1, freq="m")
         # Should be one period before target
         assert backward_date < target_date
     
